--records stats and abilities for demons
DemonBestiary_StatKey = {	--list of names to associate with stat indicies from unit queries
	[1] = "Strength",
	[2] = "Agility",
	[3] = "Stamina",
	[4] = "Intellect",
	[5] = "Spirit",
}

DemonBestiary_ResistKey = {  --list of names to associate with reisistance indicies from unit queries
	[0] = "Armor",
	[1] = "Holy",
	[2] = "Fire",
	[3] = "Nature",
	[4] = "Frost",
	[5] = "Shadow",
	[6] = "Arcane",
}

DemonBestiary_PowerTypeKey = { --list of names to assoiate with power type from unit queries
	[0] = "Mana",
	[1] = "Rage",
	[2] = "Focus",
	[3] = "Energy",
	[4] = "Happiness",
}


function DemonBestiary_Onload()
	this:RegisterEvent("PLAYER_ENTERING_WORLD")
	this:RegisterEvent("UNIT_PET")
	this:RegisterEvent("VARIBLES_LOADED")
	MyHack_Debug("demon bestiary loaded")
end

function DemonBestiary_OnEvent(event)
	if event == "PLAYER_ENTERING_WORLD" then DEFAULT_CHAT_FRAME:AddMessage("DemonBestiary loaded!") DemonBestiary_Init()
	elseif event == "UNIT_PET" then
		DemonBestiaryTimerFrame.time = GetTime()
		MyHack_Debug(event.. " " .. DemonBestiaryTimerFrame.time)
		DemonBestiaryTimerFrame:SetScript("OnUpdate",function()
			local time = GetTime()
			if time - this.time > 1 then 
				this:SetScript("OnUpdate", nil)
				if UnitExists("pet") then
					MyHack_Debug("pet exists, recording")
					DemonBestiary_RecordDemon()
					if UnitIsCharmed("pet") then
						this.profile = {realm = GetRealmName(), Player = UnitName("player"),  TimeStamp = date("%m/%d/%y %H:%M:%S"), DemonName = UnitName("pet"), DemonLevel = UnitLevel("pet"), StartShadowResist = UnitResistance("pet", 5), StartTime = time}
					end
				else
					if this.profile then
						this.profile.EndTime = time
						this.profile.Duration = this.profile.EndTime - this.profile.StartTime
						MyHack_Debug("pet existed for " .. this.profile.Duration .. " seconds, recording profile")
						table.insert(DemonBestiary_EnslaveTimes, this.profile)
						this.profile = nil
					end
				end
			end
		end)
	end
end

function DemonBestiary_Init()
	if not DemonBestiary_Grimoire then DemonBestiary_Grimoire = {} end
	if not DemonBestiary_EnslaveTimes then DemonBestiary_EnslaveTimes = {} end	
	MyHack_Debug("initilization complete")
end


function DemonBestiary_RecordDemon()
	if UnitExists("pet") then
		DemonBestiaryTooltipScanner:SetOwner(UIParent,"ANCHOR_NONE")
		local demondata = {}
		local demonname = UnitName("pet")
		local demonlevel = UnitLevel("pet")
		local location
		if not UnitIsCharmed("pet") or demonname == "Doomguard" or demonname == "Infernal" then
			location = "Player"
		else
			location = format("%s: %s", GetZoneText(), GetSubZoneText())
		end
		
		demondata.stats = {}
		table.insert(demondata.stats, format("MaxHealth: %i", UnitHealthMax("pet")))
		table.insert(demondata.stats, format("%s: %i", DemonBestiary_PowerTypeKey[UnitPowerType("pet")], UnitManaMax("pet")))
		local stats = {}
		for i=1,5 do
			table.insert(stats, format("%s:   base: %i, total: %i  +%i -%i", DemonBestiary_StatKey[i], UnitStat("pet", i)))
		end
		table.insert(demondata.stats, stats)
		local resistance = {}
		table.insert(resistance, "Resistances")
		for i=0,6 do
			table.insert(resistance, format("%s:   base: %i, total: %i  +%i -%i", DemonBestiary_ResistKey[i], UnitResistance("pet", i)))
		end
		table.insert(demondata.stats, resistance)
		
		demondata.combat = {}
		local melee = {}
		melee.LowDmg, melee.HighDmg, _, _, melee.DamagePositiveBuffed, melee.DamageNegativeDebuffed, melee.DamagePercentMod = UnitDamage("pet")
		melee.ATPbase, melee.ATPbuff, melee.ATPdebuff = UnitAttackPower("pet")
		melee.MainHandAttSpeed, _ = UnitAttackSpeed("pet")
		table.insert(demondata.combat, format("Melee Damage: %i-%i, +%i -%i;   Spd: %.2f", melee.LowDmg, melee.HighDmg, melee.DamagePositiveBuffed, melee.DamageNegativeDebuffed, melee.MainHandAttSpeed))
		table.insert(demondata.combat, format("Melee ATP: %i, +%i -%i", melee.ATPbase, melee.ATPbuff, melee.ATPdebuff))

		local ranged = {}
		ranged.Speed, ranged.LowDmg, ranged.HighDmg, ranged.PositiveBuff, ranged.NegativeDebuff, ranged.PercentModifier = UnitRangedDamage("pet")
		ranged.RangedATPbase, ranged.RangedATPbuff, ranged.RangedATPdebuff = UnitRangedAttackPower("pet")
		table.insert(demondata.combat, format("Ranged Damage: %i-%i, +%i -%i;   Spd: %.2f  mod: %.2f", ranged.LowDmg, ranged.HighDmg, ranged.PositiveBuff, ranged.NegativeDebuff, ranged.Speed, ranged.PercentModifier))
		table.insert(demondata.combat, format("Ranged ATP: %i, +%i -%i", ranged.RangedATPbase, ranged.RangedATPbuff, ranged.RangedATPdebuff))

		local abilities = {}
		for i=1,10 do
			local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(i)
			local currenttooltip
			--MyHack_Debug(i)
			if (name) and (not isToken) then 
				currenttooltip = {}
				table.insert(currenttooltip, "Tooltip:")
				DemonBestiaryTooltipScanner:ClearLines()
				DemonBestiaryTooltipScanner:SetPetAction(i)
				for j=1,DemonBestiaryTooltipScanner:NumLines() do
					local mytext = getglobal("DemonBestiaryTooltipScannerTextLeft" .. j)
					local text = mytext:GetText()
					--MyHack_Debug(text)
					table.insert(currenttooltip, text)
				end
				abilities[i] = {}
				abilities[i][1] = format("%s %s", name, subtext)
				if autoCastAllowed then abilities[i][1] = abilities[i][1] .. ", auto cast allowed" end
				abilities[i][2] = currenttooltip 
			end
		end
		demondata.abilities = abilities	
		if not DemonBestiary_Grimoire[location] then DemonBestiary_Grimoire[location] = {} end
		if not DemonBestiary_Grimoire[location][demonname] then DemonBestiary_Grimoire[location][demonname] = {} end
		if DemonBestiary_Grimoire[location][demonname][demonlevel] then
			demondata.count = DemonBestiary_Grimoire[location][demonname][demonlevel].count + 1
		else 
			demondata.count = 1 
		end
		DemonBestiary_Grimoire[location][demonname][demonlevel] = demondata
	end
	DemonBestiaryTooltipScanner:SetOwner(UIParent,"ANCHOR_NONE")
end

function MyHack_Debug(msg)
	ChatFrame5:AddMessage(msg)
end

